// This file is part of Pushup Buddy.
// Copyright 2010 Fredrik Portstrom
//
// Pushup Buddy is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// Pushup Buddy is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Pushup Buddy. If not, see
// <http://www.gnu.org/licenses/>.

package org.example.pushupbuddy;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.content.SharedPreferences;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.os.SystemClock;
import android.preference.PreferenceManager;
import android.speech.tts.TextToSpeech;
import java.util.List;
import java.util.Locale;

public class MainService extends Service {
    MainActivity mainActivity;
    Workout workout;
    private boolean mPreferenceCountLoud;
    private boolean mPreferenceTellProgress;
    private boolean mPreferenceRestReminder;
    private boolean mPreferenceOverdueReminder;
    private boolean mSpeechInitialized;
    private int mPreferenceMinPushupTime;
    private int mOverdueTime;
    private int mPreferenceOverdueInterval;
    private int mPreferenceRestDuration;
    private int mPreferenceRestTimeOut;
    private int mPushupEndTime;
    private int mPushupStartTime = -1;
    private int mCreatedAt;
    private long mStartTime = -1;
    private Handler mHandler = new Handler();
    private IBinder mBinder = new LocalBinder();
    private Runnable mRunnable;
    private SensorManager mSensorManager;
    private Set mCurrentSet;
    private SharedPreferences mPreferences;
    private TextToSpeech mTextToSpeech;

    @Override
    public void onCreate() {
	mPreferences = PreferenceManager.getDefaultSharedPreferences(this);
	mPreferences.registerOnSharedPreferenceChangeListener(mOnSharedPreferenceChangeListener);
	loadPreferences();
	Notification notification = new Notification();
	notification.flags = Notification.FLAG_ONGOING_EVENT;
	notification.icon = R.drawable.icon;
	notification.setLatestEventInfo(this, "Pushups", null, PendingIntent.getActivity(this, 0, new Intent(this, MainActivity.class), 0));
	((NotificationManager) getSystemService(NOTIFICATION_SERVICE)).notify(0, notification);
	//startForeground(0, notification);
	mSensorManager = (SensorManager)getSystemService(SENSOR_SERVICE);
	mTextToSpeech = new TextToSpeech(this, new TextToSpeech.OnInitListener() {
		@Override
		public void onInit(int status) {
		    if (status == TextToSpeech.SUCCESS) {
			mSpeechInitialized = true;
			mTextToSpeech.setLanguage(Locale.ENGLISH);
		    }
		}
	    });
	List<Sensor> sensors = mSensorManager.getSensorList(Sensor.TYPE_PROXIMITY);
	for (Sensor sensor : sensors) {
	    mSensorManager.registerListener(mSensorEventListener, sensor, SensorManager.SENSOR_DELAY_GAME);
	}
    }

    @Override
    public void onDestroy() {
	if (workout != null) {
	    Database database = new Database(this);
	    long id = database.database.compileStatement("INSERT INTO workouts (created_at, pushup_count, set_count, time_elapsed, data) VALUES (" + mCreatedAt + ", " + workout.pushupCount + ", " + workout.setCount + ", " + workout.timeElapsed + ", '" + workout.serialize() + "')").executeInsert();
	    database.close();

	    if (mainActivity != null) {
		mainActivity.workoutId = id;
	    }
	}

	//stopForeground(true);
	((NotificationManager) getSystemService(NOTIFICATION_SERVICE)).cancel(0);
	mHandler.removeCallbacks(mRunnable);
	mSensorManager.unregisterListener(mSensorEventListener);
	mPreferences.unregisterOnSharedPreferenceChangeListener(mOnSharedPreferenceChangeListener);
	mSpeechInitialized = false;
	mTextToSpeech.shutdown();
	mTextToSpeech = null;
    }

    @Override
    public IBinder onBind(Intent intent) {
	return mBinder;
    }

    private void loadPreferences() {
	mPreferenceCountLoud = mPreferences.getBoolean("count_loud", true);
	mPreferenceTellProgress = mPreferences.getBoolean("tell_progress", true);
	mPreferenceRestReminder = mPreferences.getBoolean("rest_reminder", true);
	mPreferenceOverdueReminder = mPreferences.getBoolean("overdue_reminder", true);
	mPreferenceMinPushupTime = mPreferences.getInt("min_pushup_time", 500);
	mPreferenceRestTimeOut = mPreferences.getInt("set_time_out", 6000);
	mPreferenceRestDuration = mPreferences.getInt("rest_duration", 90000);
	mPreferenceOverdueInterval = mPreferences.getInt("overdue_interval", 30000);
    }

    private void callDelayed(Runnable runnable, int delay) {
	mRunnable = runnable;
	mHandler.postDelayed(runnable, delay);
    }

    private String formatTime(long time) {
	time = (time + 500) / 1000;
	return time == 1 ? "1 second"
	    : time < 60 ? time + " seconds"
	    : time < 61 ? "1 minute"
	    : time < 62 ? "1 minute and 1 second"
	    : time < 120 ? "1 minute and " + time % 60 + " seconds"
	    : time % 60 < 1 ? time / 60 + " minutes"
	    : time % 60 < 2 ? time / 60 + " minutes and 1 second"
	    : time / 60 + " minutes and " + time % 60 + " seconds";
    }

    private SharedPreferences.OnSharedPreferenceChangeListener mOnSharedPreferenceChangeListener = new SharedPreferences.OnSharedPreferenceChangeListener() {
	    @Override
	    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
		loadPreferences();
	    }
	};

    private SensorEventListener mSensorEventListener = new SensorEventListener() {
	    @Override
	    public void onAccuracyChanged(Sensor sensor, int accuracy) {
	    }

	    @Override
	    public void onSensorChanged(SensorEvent event) {
		mHandler.removeCallbacks(mRunnable);

		if (event.values[0] < 1) {
		    if (mPushupStartTime < 0) {
			if (mStartTime < 0) {
			    mCreatedAt = (int)(System.currentTimeMillis() / 1000);
			    mPushupStartTime = 0;
			    mStartTime = SystemClock.elapsedRealtime();
			} else {
			    mPushupStartTime = (int)(SystemClock.elapsedRealtime() - mStartTime);
			}
		    }
		} else if(mPushupStartTime >= 0) {
		    mPushupEndTime = (int)(SystemClock.elapsedRealtime() - mStartTime);
		    callDelayed(mCountRunnable, mPreferenceMinPushupTime);
		}
	    }
	};

    private Runnable mCountRunnable = new Runnable() {
	    public void run() {
		if (workout == null) {
		    workout = new Workout();
		}

		if (mCurrentSet == null) {
		    workout.setCount++;
		    mCurrentSet = new Set();
		    mCurrentSet.startTime = mPushupStartTime;
		    workout.sets.add(mCurrentSet);
		}

		mPushupStartTime = -1;
		mCurrentSet.endTime = mPushupEndTime;
		mCurrentSet.pushupCount++;
		workout.pushupCount++;
		workout.timeElapsed = mPushupEndTime;

		if (mainActivity != null) {
		    mainActivity.update(workout);
		}

		callDelayed(mEndSetRunnable, mPreferenceRestTimeOut);

		if (mPreferenceCountLoud && mSpeechInitialized) {
		    mTextToSpeech.speak("" + mCurrentSet.pushupCount, 0, null);
		}
	    }
	};

    private Runnable mEndSetRunnable = new Runnable() {
	    public void run() {
		if (mPreferenceTellProgress && mSpeechInitialized) {
		    String s = "Set " + workout.setCount + " done in " + formatTime(mPushupEndTime - mCurrentSet.startTime) + ". " + (mCurrentSet.pushupCount > 1 ? mCurrentSet.pushupCount + " pushups" : "1 pushup");
		    if (workout.setCount > 1) {
			s += " in this set. " + workout.pushupCount + " pushups total. Total time " + formatTime(mPushupEndTime);
		    }
		    mTextToSpeech.speak(s, 0, null);
		}

		mCurrentSet = null;

		if (mPreferenceRestDuration > 0) {
		    callDelayed(mBeginSetRunnable, mPreferenceRestDuration);
		}
	    }
	};

    private Runnable mBeginSetRunnable = new Runnable() {
	    public void run() {
		if (mPreferenceRestReminder && mSpeechInitialized) {
		    mTextToSpeech.speak("Begin next set", 0, null);
		}

		if (mPreferenceOverdueInterval > 0) {
		    mOverdueTime = mPreferenceOverdueInterval;
		    callDelayed(mOverdueRunnable, mPreferenceOverdueInterval);
		}
	    }
	};

    private Runnable mOverdueRunnable = new Runnable() {
	    public void run() {
		if (mPreferenceOverdueReminder && mSpeechInitialized) {
		    mTextToSpeech.speak("" + formatTime(mOverdueTime) + " overdue for next set", 0, null);
		}

		if (mPreferenceOverdueInterval > 0) {
		    mOverdueTime += mPreferenceOverdueInterval;
		    callDelayed(mOverdueRunnable, mPreferenceOverdueInterval);
		}
	    }
	};

    class LocalBinder extends Binder {
	MainService getService() {
	    return MainService.this;
	}
    }
}
